#include <avr/io.h>

#define MCP79410_RTC_WRITE_ADDRESS 0xDE
#define MCP79410_RTC_READ_ADDRESS 0xDF
#define MCP79410_EEPROM_WRITE_ADDRESS 0xAE
#define MCP79410_EEPROM_READ_ADDRESS 0xAF

#define MCP79410_RTCSEC 0x00
#define MCP79410_RTCMIN 0x01
#define MCP79410_RTCHOUR 0x02
#define MCP79410_RTCWKDAY 0x03
#define MCP79410_RTCDATE 0x04
#define MCP79410_RTCMTH 0x05
#define MCP79410_RTCYEAR 0x06
#define MCP79410_CONTROL 0x07
#define MCP79410_OSCTRIM 0x08
#define MCP79410_EEUNLOCK 0x09
#define MCP79410_ALM0SEC 0x0A
#define MCP79410_ALM0WKDAY 0x0D
#define MCP79410_ALM1SEC 0x11

#define MCP79410_VBATEN 3
#define MCP79410_ST 7
#define MCP79410_OUT 7
#define MCP79410_ALM0MSK0 4
#define MCP79410_ALM0MSK1 5
#define MCP79410_ALM0MSK2 6
#define MCP79410_ALM0EN 4

void rtcReadRegister(uint8_t reg, uint8_t * data);
void rtcWriteRegister(uint8_t reg, uint8_t data);
void rtcReadRegisters(uint8_t reg, uint8_t * data, uint8_t size);
void rtcWriteRegisters(uint8_t reg, uint8_t * data, uint8_t size);
uint8_t fromBCD(uint8_t val);
uint8_t toBCD(uint8_t val);
void eepromReadByte(uint8_t reg, uint8_t * data);
void eepromWriteByte(uint8_t reg, uint8_t data);
void eepromReadBytes(uint8_t reg, uint8_t * data, uint8_t size);
void eepromWriteBytes(uint8_t reg, uint8_t * data, uint8_t size);
uint8_t eepromIsBusy(void);
void eepromWriteProtected(uint8_t reg, uint8_t * data, uint8_t size);

typedef struct {
	uint8_t RTCSEC;
	uint8_t RTCMIN;
	uint8_t RTCHOUR;
	uint8_t RTCWKDAY;
	uint8_t RTCDATE;
	uint8_t RTCMTH;
	uint8_t RTCYEAR;
} RtcDateTimeRegisters;

typedef struct {
	uint8_t ALM0SEC;
	uint8_t ALM0MIN;
	uint8_t ALM0HOUR;
	uint8_t ALM0WKDAY;
	uint8_t ALM0DATE;
	uint8_t ALM0MTH;
} RtcAlarmRegisters;

typedef struct {
	uint8_t seconds;
	uint8_t minutes;
	uint8_t hours;
	uint8_t dayOfWeek;
	uint8_t dayOfMonth;
	uint8_t month;
	uint16_t year;
} DateTime;

void rtcReadDateTime(DateTime * dateTime);
void rtcWriteDateTime(DateTime * dateTime);
void rtcWriteMinuteAlarm(uint8_t alarmNumber);